@@ -78,8 +78,8 @@ pango_cairo_font_map_get_type (void)
  * You generally should only use the #PangoFontMap and
  * #PangoCairoFontMap interfaces on the returned object.
  * 
- * Return value: the newly created fontmap object. Free
- * with g_object_unref().
+ * Return value: the newly allocated #PangoFontMap, which should
+ *               be freed with g_object_unref().
  *
  * Since: 1.10
  **/
@@ -397,7 +397,7 @@ pango_cairo_context_set_font_options (PangoContext               *context,
  *   if no options have been set. This value is owned by the context
  *   and must not be modified or freed.
  **/
-const cairo_font_options_t *
+G_CONST_RETURN cairo_font_options_t *
 pango_cairo_context_get_font_options (PangoContext *context)
 {
   PangoCairoContextInfo *info;
@@ -423,7 +423,7 @@ pango_cairo_context_get_font_options (PangoContext *context)
  * Return value: the combined set of font options. This value is owned
  * by the context and must not be modified or freed.
  **/
-const cairo_font_options_t *
+G_CONST_RETURN cairo_font_options_t *
 _pango_cairo_context_get_merged_font_options (PangoContext *context)
 {
   PangoCairoContextInfo *info = get_context_info (context, TRUE);
